@@ -6,6 +6,7 @@ use alloc::vec;
 use alloc::vec::Vec;
 use core::convert::TryInto;
 use core::mem::{size_of, transmute_copy};
+use postcard::{to_allocvec, from_bytes};
 
 use crate::event_processor::EventProcessor;
 use crate::extractor::*;
@@ -14,7 +15,7 @@ use crate::filter::*;
 use crate::matcher::Matcher;
 use crate::producer::state::*;
 use crate::producer::*;
-use crate::Value;
+use crate::StateValue;
 
 macro_rules! impl_item_read {
     ($item_code:expr, $item_type:ty, $data:expr, $offset:expr, $provided_code:expr) => {
@@ -75,15 +76,15 @@ macro_rules! write_integer_to_vec {
 }
 
 #[derive(Debug)]
-pub struct Config<'a> {
-    pub initial_state: BTreeMap<u32, Value<'a>>,
+pub struct Config {
+    pub initial_state: BTreeMap<u32, StateValue>,
     pub event_processors: Vec<EventProcessor>,
 }
 
 #[derive(Debug, PartialEq)]
 pub enum ConfigSerializerError {
     WrongSize,
-    TriedToSaveReferenceValue,
+    PostcardError(postcard::Error),
     UnknownExtractor,
     UnknownFilter,
     UnknownProducer,
@@ -100,14 +101,12 @@ impl ConfigSerializer {
         for state in config.initial_state.iter() {
             write_integer_to_vec!(data, *state.0, u32);
 
-            if let Value::Reference(_) = state.1 {
-                return Err(ConfigSerializerError::TriedToSaveReferenceValue);
-            }
+            let serialized_state = to_allocvec(state.1)?;
 
-            unsafe {
-                for byte in transmute_copy::<Value, [u8; size_of::<Value>()]>(state.1).iter() {
-                    data.push(*byte);
-                }
+            write_integer_to_vec!(data, serialized_state.len() as u32, u32);
+
+            for byte in serialized_state.iter() {
+                data.push(*byte);
             }
         }
 
@@ -128,7 +127,7 @@ impl ConfigSerializer {
         Ok(data)
     }
 
-    pub fn deserialize<'a, 'b>(data: &'a Vec<u8>) -> Result<Config<'b>, ConfigSerializerError> {
+    pub fn deserialize(data: &Vec<u8>) -> Result<Config, ConfigSerializerError> {
         let mut offset = 0;
 
         let initial_state_count = read_integer_from_vec!(data, offset, u32);
@@ -136,14 +135,10 @@ impl ConfigSerializer {
 
         for _ in 0..initial_state_count {
             let state_index = read_integer_from_vec!(data, offset, u32);
-            let state_value = unsafe {
-                const SIZE: usize = size_of::<Value>();
-                let item = transmute_copy::<[u8; SIZE], Value>(
-                    data[offset..offset + SIZE].try_into().unwrap(),
-                );
-                offset += SIZE;
-                item
-            };
+            let serialized_state_len = read_integer_from_vec!(data, offset, u32) as usize;
+
+            let state_value = from_bytes(&data[offset..offset + serialized_state_len])?;
+            offset += serialized_state_len;
 
             initial_state.insert(state_index, state_value);
         }
@@ -377,3 +372,104 @@ impl ConfigSerializer {
         Err(ConfigSerializerError::UnknownProducer)
     }
 }
+
+impl From<postcard::Error> for ConfigSerializerError {
+    fn from(err: postcard::Error) -> Self {
+        Self::PostcardError(err)
+    }
+}
+
+#[cfg(test)]
+mod tests {
+    use super::*;
+
+    #[test]
+    fn serialize_config_serializer_test() {
+        let mut initial_state = BTreeMap::new();
+        initial_state.insert(0, StateValue::U8(0xff));
+
+        let mut event_processors = vec![];
+        event_processors.push(EventProcessor {
+            matchers: vec![Matcher {
+                extractor: Box::new(EventCodeExtractor::new()),
+                filter: Box::new(U16IsEqualFilter::new(0xff)),
+            }],
+            extractor: Box::new(NoneExtractor::new()),
+            producer: Box::new(BcmChangeBrightnessStateProducer::new(0xff, 0xff, 0)),
+        });
+
+        let config = Config {
+            initial_state,
+            event_processors,
+        };
+
+        let data = ConfigSerializer::serialize(&config).unwrap();
+
+        let expected_data = vec![
+            0x00, 0x00, 0x00, 0x01, // initial state count
+            0x00, 0x00, 0x00, 0x00, // state_index
+            0x00, 0x00, 0x00, 0x02,  // state_value len
+            0x00, 0xff, // state_value
+            0x00, 0x00, 0x00, 0x01, // event processor count
+            0x00, 0x00, 0x00, 0x01, // matcher count
+            0x00, 0x01, // EVENT_CODE_EXTRACTOR_CODE
+            0x00, 0x01, // U16_IS_EQUAL_FILTER_CODE
+            0xff, 0x00, // value
+            0x00, 0x00, // NONE_EXTRACTOR_CODE
+            0x00, 0x02, // BCM_CHANGE_BRIGHTNESS_STATE_PRODUCER_CODE
+            0xff, 0x00, // bcm_address
+            0xff, 0x00, // channel
+            0x00, 0x00, 0x00, 0x00, // state_index
+        ];
+
+        assert_eq!(data, expected_data);
+    }
+
+    #[test]
+    fn deserialize_config_serializer_wrong_size_test() {
+        let data = vec![];
+
+        let err = ConfigSerializer::deserialize(&data).unwrap_err();
+
+        assert_eq!(ConfigSerializerError::WrongSize, err);
+    }
+
+    #[test]
+    fn deserialize_config_serializer_empty_test() {
+        let data = vec![
+            0x00, 0x00, 0x00, 0x00, // initial state count
+            0x00, 0x00, 0x00, 0x00, // event processor count
+        ];
+
+        let config = ConfigSerializer::deserialize(&data).unwrap();
+
+        assert_eq!(config.initial_state.len(), 0);
+        assert_eq!(config.event_processors.len(), 0);
+    }
+
+    #[test]
+    fn deserialize_config_serializer_test() {
+        let data = vec![
+            0x00, 0x00, 0x00, 0x01, // initial state count
+            0x00, 0x00, 0x00, 0x00, // state_index
+            0x00, 0x00, 0x00, 0x02, // state_value len
+            0x00, 0xff, // state_value
+            0x00, 0x00, 0x00, 0x01, // event processor count
+            0x00, 0x00, 0x00, 0x01, // matcher count
+            0x00, 0x01, // EVENT_CODE_EXTRACTOR_CODE
+            0x00, 0x01, // U16_IS_EQUAL_FILTER_CODE
+            0xff, 0x00, // value
+            0x00, 0x00, // NONE_EXTRACTOR_CODE
+            0x00, 0x02, // BCM_CHANGE_BRIGHTNESS_STATE_PRODUCER_CODE
+            0xff, 0x00, // bcm_address
+            0xff, 0x00, // channel
+            0x00, 0x00, 0x00, 0x00, // state_index
+        ];
+
+        let config = ConfigSerializer::deserialize(&data).unwrap();
+
+        assert_eq!(config.initial_state.len(), 1);
+        assert_eq!(*config.initial_state.get(&0).unwrap(), StateValue::U8(0xff));
+        assert_eq!(config.event_processors.len(), 1);
+    }
+}
