@@ -1,5 +1,7 @@
 #![no_std]
 
+use serde::{Serialize, Deserialize};
+
 use ross_protocol::packet::Packet;
 
 pub mod config;
@@ -10,19 +12,18 @@ pub mod matcher;
 pub mod producer;
 pub mod state;
 
-#[cfg(test)]
-mod test;
+#[derive(Serialize, Deserialize, Debug, PartialEq)]
+pub enum StateValue {
+    U8(u8),
+    U16(u16),
+    U32(u32),
+}
 
 #[derive(Debug, PartialEq)]
-pub enum Value<'a> {
+pub enum ExtractorValue<'a> {
     None,
     U8(u8),
     U16(u16),
     U32(u32),
-    Reference(ReferenceValue<'a>),
-}
-
-#[derive(Debug, PartialEq)]
-pub enum ReferenceValue<'a> {
     Packet(&'a Packet),
 }
